@@ -525,9 +525,6 @@ process_backend_health_check_failure(int health_check_node_id, int retrycnt)
 	int health_check_max_retries = pool_config->parallel_mode ?
 			(NUM_BACKENDS - 1) : pool_config->health_check_max_retries;
 
-	pool_signal(SIGALRM, SIG_IGN);	/* Cancel timer */
-	CLEAR_ALARM;
-
 	if (health_check_max_retries > 0 && retrycnt <= health_check_max_retries)
 	{
 		/* Keep retrying and sleep a little in between */
@@ -1430,6 +1427,17 @@ static void failover(void)
 	}
 	Req_info->switching = true;
 	switching = 1;
+
+	/* Perform failover with health check alarm
+	 * disabled
+	 */
+	if (pool_config->health_check_timeout > 0)
+	{
+		pool_signal(SIGALRM, SIG_IGN);
+		CLEAR_ALARM;
+		health_check_timer_expired = 0;
+	}
+
 	for(;;)
 	{
 		POOL_REQUEST_KIND reqkind;
@@ -2030,7 +2038,7 @@ do_health_check(bool use_template_db, volatile int *health_check_node_id)
 		ereport(DEBUG1,
 			(errmsg("Trying to make persistent DB connection to backend node %d having status %d", i, bkinfo->backend_status)));
 
-		slot = make_persistent_db_connection(bkinfo->backend_hostname,
+		slot = make_persistent_db_connection(i, bkinfo->backend_hostname,
 											 bkinfo->backend_port,
 											 dbname,
 											 pool_config->health_check_user,
@@ -2578,7 +2586,7 @@ static bool
 
 	*is_standby = false;
 
-	s = make_persistent_db_connection_noerror(bkinfo->backend_hostname,
+	s = make_persistent_db_connection_noerror(backend_no, bkinfo->backend_hostname,
 										  bkinfo->backend_port,
 										  "postgres",
 										  pool_config->sr_check_user,
