@@ -253,19 +253,28 @@ static int init_ssl_ctx(POOL_CONNECTION *cp, enum ssl_conn_type conntype) {
 	char *cacert = NULL, *cacert_dir = NULL;
 
 	/* initialize SSL members */
-	cp->ssl_ctx = SSL_CTX_new(TLSv1_method());
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+		cp->ssl_ctx = SSL_CTX_new(TLS_method());
+#else
+		cp->ssl_ctx = SSL_CTX_new(SSLv23_method());
+#endif
+
 	SSL_RETURN_ERROR_IF( (! cp->ssl_ctx), "SSL_CTX_new" );
+	/*
+	 * Disable OpenSSL's moving-write-buffer sanity check, because it
+	 * causes unnecessary failures in nonblocking send cases.
+	 */
+	SSL_CTX_set_mode(cp->ssl_ctx, SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
 
 	if ( conntype == ssl_conn_serverclient) {
-		error = SSL_CTX_use_certificate_file(cp->ssl_ctx,
-		                                     pool_config->ssl_cert,
-		                                     SSL_FILETYPE_PEM);
-		SSL_RETURN_ERROR_IF( (error <= 0), "Loading SSL certificate");
+		error = SSL_CTX_use_certificate_chain_file(cp->ssl_ctx,
+		                                     pool_config->ssl_cert);
+		SSL_RETURN_ERROR_IF( (error != 1), "Loading SSL certificate");
 
 		error = SSL_CTX_use_PrivateKey_file(cp->ssl_ctx,
 		                                    pool_config->ssl_key,
 		                                    SSL_FILETYPE_PEM);
-		SSL_RETURN_ERROR_IF( (error <= 0), "Loading SSL private key");
+		SSL_RETURN_ERROR_IF( (error != 1), "Loading SSL private key");
 	} else {
 		/* set extra verification if ssl_ca_cert or ssl_ca_cert_dir are set */
 		if (strlen(pool_config->ssl_ca_cert))
@@ -274,10 +283,10 @@ static int init_ssl_ctx(POOL_CONNECTION *cp, enum ssl_conn_type conntype) {
 			cacert_dir = pool_config->ssl_ca_cert_dir;
     
 		if ( cacert || cacert_dir ) {
-			error = (!SSL_CTX_load_verify_locations(cp->ssl_ctx,
+			error = SSL_CTX_load_verify_locations(cp->ssl_ctx,
 			                                        cacert,
-			                                        cacert_dir));
-			SSL_RETURN_ERROR_IF(error, "SSL verification setup");
+			                                        cacert_dir);
+			SSL_RETURN_ERROR_IF((error != 1), "SSL verification setup");
 			SSL_CTX_set_verify(cp->ssl_ctx, SSL_VERIFY_PEER, NULL);
 		}
 	}
